//------------------------------------------------------------------------
//
// Author      : Jan Horn
// Email       : jhorn@global.co.za
// Website     : http://www.sulaco.co.za
//               http://home.global.co.za/~jhorn
// Date        : 23 February 2002
// Version     : 1.0
// Description : Delphi Forms Application Template
//
//------------------------------------------------------------------------
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, OpenGL,
  ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    ScrollBar1: TScrollBar;
    Label1: TLabel;
    ScrollBar2: TScrollBar;
    Label2: TLabel;
    ScrollBar3: TScrollBar;
    Label3: TLabel;
    Label4: TLabel;
    CheckBox1: TCheckBox;
    Timer1: TTimer;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    rc : HGLRC;    // Rendering Context
    dc  : HDC;     // Device Context
    ElapsedTime, AppStart, LastTime : DWord;  // Timing variables
    procedure glDraw;
    procedure Idle(Sender: TObject; var Done: Boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

var PlayerX,PlayerY,PlayerZ:single; // koordinati igroka
var CameraX,CameraY,CameraZ:single; // koordinati kameri

var PlayerPhi:single; // ugol povorota igroka
var CameraPhi:single; // ugol povorota kameri

var CameraDist:single=50; // rasstoyanie do kameri

var WalkSpeed:single=1;
var RotateSpeed:single=6;
{------------------------------------------------------------------}
{  Function to draw the actual scene                               }
{------------------------------------------------------------------}
procedure TForm1.glDraw();
var i:integer;
begin
 

  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);    // Clear The Screen And The Depth Buffer
  glLoadIdentity();                                       // Reset The View

  CameraPhi:=ScrollBar1.Position;
  CameraDist:=ScrollBar2.Position;

  CameraX:=PlayerX+CameraDist*sin(CameraPhi*3.14/180);
  CameraY:=ScrollBar3.Position;
  CameraZ:=Playerz+CameraDist*cos(CameraPhi*3.14/180);

  gluLookAt(CameraX,CameraY,CameraZ,
            PlayerX,PlayerY,PlayerZ,
            0,1,0);

  // setka
  glColor3f(0.5,0.5,0.5);
  glBegin(GL_LINES);
   for i := 0 to 10 do
    begin
     glVertex(-50,0,-50+i*10); glVertex(50,0,-50+i*10);
     glVertex(-50+i*10,0,-50); glVertex(-50+i*10,0,50);
    end;
  glEnd();


  // z-fight
  PlayerY:=0.5;

  // treugolnik Player
  glTranslatef(PlayerX,PlayerY,PlayerZ);
  glRotatef(PlayerPhi,0,1,0);
  glBegin(GL_TRIANGLES);
    glColor3f(1,1,1);glVertex( 0,0, 7);
    glColor3f(1,0,0);glVertex(-5,0,-5);
    glColor3f(1,0,0);glVertex(+5,0,-5);
  glEnd();

  // sistema koordinat
  glColor3f(1,1,1);
  glBegin(GL_LINES);
  glVertex(0,1, 0); glVertex(5,1, 0); // X
  glVertex(0,1, 0); glVertex(0,5, 0); // Y
  glVertex(0,1, 0); glVertex(0,1, 5); // Z
  glEnd();


end;


{------------------------------------------------------------------}
{  Initialise OpenGL                                               }
{------------------------------------------------------------------}
procedure glInit();
begin
  glClearColor(0.0, 0.0, 0.0, 0.0); 	   // Black Background
  glShadeModel(GL_SMOOTH);                 // Enables Smooth Color Shading
  glClearDepth(1.0);                       // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                 // Enable Depth Buffer
  glDepthFunc(GL_LESS);		           // The Type Of Depth Test To Do

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //Realy Nice perspective calculations
end;


{------------------------------------------------------------------}
{  Create the form and initialist openGL                           }
{------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
var pfd : TPIXELFORMATDESCRIPTOR;
    pf  : Integer;
begin
  // OpenGL initialisieren
  dc:=GetDC(Panel1.Handle);

  // PixelFormat
  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
  pfd.iPixelType:=PFD_TYPE_RGBA;      // PFD_TYPE_RGBA or PFD_TYPEINDEX
  pfd.cColorBits:=32;

  pf :=ChoosePixelFormat(dc, @pfd);   // Returns format that most closely matches above pixel format
  SetPixelFormat(dc, pf, @pfd);

  rc :=wglCreateContext(dc);    // Rendering Context = window-glCreateContext
  wglMakeCurrent(dc,rc);        // Make the DC (Form1) the rendering Context

  // Initialist GL environment variables
  glInit;
  Panel1Resize(sender);    // sets up the perspective
  AppStart :=GetTickCount();

  // when the app has spare time, render the GL scene
  Application.OnIdle := Idle;
end;


{------------------------------------------------------------------}
{  Release rendering context when form gets detroyed               }
{------------------------------------------------------------------}
procedure TForm1.FormDestroy(Sender: TObject);
begin
  wglMakeCurrent(0,0);
  wglDeleteContext(rc);
end;


{------------------------------------------------------------------}
{  Application onIdle event                                        }
{------------------------------------------------------------------}
procedure TForm1.Idle(Sender: TObject; var Done: Boolean);
begin
  Done := FALSE;

  LastTime :=ElapsedTime;
  ElapsedTime :=GetTickCount() - AppStart;      // Calculate Elapsed Time
  ElapsedTime :=(LastTime + ElapsedTime) DIV 2; // Average it out for smoother movement

  glDraw();                         // Draw the scene
  SwapBuffers(DC);                  // Display the scene
end;


{------------------------------------------------------------------}
{  If the panel resizes, reset the GL scene                        }
{------------------------------------------------------------------}
procedure TForm1.Panel1Resize(Sender: TObject);
begin
  glViewport(0, 0, Panel1.Width, Panel1.Height);    // Set the viewport for the OpenGL window
  glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
  glLoadIdentity();                   // Reset View
  gluPerspective(45.0, Panel1.Width/Panel1.Height, 1.0, 500.0);  // Do the perspective calculations. Last value = max clipping depth

  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
end;


procedure TForm1.Timer1Timer(Sender: TObject);
begin
 if CheckBox1.Checked then
  ScrollBar1.Position:=ScrollBar1.Position+1;

  if ScrollBar1.Position=360 then ScrollBar1.Position:=0;
end;

{------------------------------------------------------------------}
{  Monitors all keypress events for the app                        }
{------------------------------------------------------------------}
procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then
    Close;

  if (Key= 'w') or (Key='W') then
   begin
    PlayerX:=PlayerX+WalkSpeed*sin(PlayerPhi*3.14/180);
    PlayerZ:=PlayerZ+WalkSpeed*cos(PlayerPhi*3.14/180);
   end;

  if (Key= 'd') or (Key='D') then
   begin
    PlayerPhi:=PlayerPhi-RotateSpeed;
   end;

  if (Key= 'a') or (Key='A') then
   begin
    PlayerPhi:=PlayerPhi+RotateSpeed;
   end;


   
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Close;
end;

end.
